from typing import List
from codetocad.core.dimension import Dimension
from codetocad.core.math_utils import linspace
from codetocad.core.point import Point


def _calculate_ellipse_point(x: Dimension, a: Dimension, b: Dimension) -> Point:
    """
    Using the equation x^2/a^2 + y^2/b^2 = 1, we will calculate a point on an elliptical path.
    """
    a_squared = a.raise_power(2)
    b_squared = b.raise_power(2)
    x_squared = x.raise_power(2)

    # y^2/b^2 = 1 - x^2/a^2
    y_squared_div_b_squared = Dimension(1, x.unit) - (x_squared / a_squared)
    y_squared = y_squared_div_b_squared * b_squared
    return Point(x=x, y=y_squared.raise_power(1 / 2), z=Dimension.zero())


def get_ellipse_points_in_fourth_quadrant(
    radius_x: Dimension, radius_y: Dimension, quadrant_resolution: int
) -> List[Point]:
    """
    Returns linspaced points making up an ellipse's fourth quadrant.
    """

    points: List[Point] = []

    first_point = Point(radius_x * -1, Dimension.zero(), Dimension.zero())
    middle_point = Point(Dimension.zero(), radius_y, Dimension.zero())

    for x in linspace(first_point.x.value, middle_point.x.value, quadrant_resolution):
        points.append(_calculate_ellipse_point(x=Dimension(x), a=radius_x, b=radius_y))

    return points


def get_ellipse_points(radius_x: Dimension, radius_y: Dimension, resolution: int):
    """
    Returns linspaced points making up a complete ellipse.
    NOTE: The first and last points are the same to denote a closed shape.
    """
    quadrant_resolution = int(resolution / 4) + 1

    fourth_quadrant = get_ellipse_points_in_fourth_quadrant(
        radius_x, radius_y, quadrant_resolution
    )

    first_quadrant = [
        Point(point.x * -1, point.y, point.z) for point in fourth_quadrant[-2::-1]
    ]

    fourth_first_quadrants = fourth_quadrant + first_quadrant

    second_third_quadrants = [
        Point(point.x, point.y * -1, point.z)
        for point in fourth_first_quadrants[-2::-1]
    ]

    return fourth_first_quadrants + second_third_quadrants



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import EntityInterface


class CameraInterface(EntityInterface, metaclass=ABCMeta):
    """Manipulate a camera object."""

    name: str
    description: Optional[str] = None

    @abstractmethod
    def __init__(
        self, name: str, description: Optional[str] = None, native_instance=None
    ):
        super().__init__(
            name=name, description=description, native_instance=native_instance
        )
        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def create_perspective(self):
        """
        Create a perspective camera in the scene.
        """

        print(
            "create_perspective is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_orthogonal(self):
        """
        Create an orthogonal camera in the scene.
        """

        print(
            "create_orthogonal is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_focal_length(self, length: float):
        """
        Set the focal length of the camera.
        """

        print(
            "set_focal_length is called in an abstract method. Please override this method."
        )
        return self



from codetocad.core.dimension import Dimension
from codetocad.core.point import Point


class Dimensions:
    def __init__(self, x: Dimension, y: Dimension, z: Dimension) -> None:
        self.point = Point(x, y, z)

    def to_list(self):
        return self.point.to_list()

    @staticmethod
    def from_point(point: Point) -> "Dimensions":
        return Dimensions(point.x, point.y, point.z)

    @classmethod
    def from_list(cls, dimensions_list: list[Dimension]):
        assert (
            len(dimensions_list) == 3
        ), "Dimensions list must contain three Dimensions."
        return cls(dimensions_list[0], dimensions_list[1], dimensions_list[2])

    def __eq__(self, other) -> bool:
        return self.point == other.point

    def __add__(self, other):
        point = self.point + other.point
        return Dimensions(point.x, point.y, point.z)

    def __sub__(self, other):
        point = self.point - other.point
        return Dimensions(point.x, point.y, point.z)

    def __mul__(self, other):
        point = self.point * other.point
        return Dimensions(point.x, point.y, point.z)

    def __truediv__(self, other):
        point = self.point / other.point
        return Dimensions(point.x, point.y, point.z)

    def __floordiv__(self, other):
        point = self.point // other.point
        return Dimensions(point.x, point.y, point.z)

    def __mod__(self, other):
        point = self.point % other.point
        return Dimensions(point.x, point.y, point.z)

    def __divmod__(self, other):
        point = divmod(self.point, other.point)
        return Dimensions(point.x, point.y, point.z)

    def __pow__(self, other, mod=None):
        point = pow(self.point, other.point)
        return Dimensions(point.x, point.y, point.z)

    def __abs__(self):
        point = abs(self.point)
        return Dimensions(point.x, point.y, point.z)

    def __copy__(self):
        return self.__deepcopy__()

    def __deepcopy__(self):
        return Dimensions(self.point.x, self.point.y, self.point.z)

    def copy(self):
        return self.__deepcopy__()

    @property
    def x(self):
        return self.point.x

    @property
    def y(self):
        return self.point.y

    @property
    def z(self):
        return self.point.z

    @property
    def radius(self):
        return self.x

    @property
    def width(self):
        return self.x

    @property
    def length(self):
        return self.y

    @property
    def height(self):
        return self.z

    def __getitem__(self, key):
        if key == 0:
            return self.x
        if key == 1:
            return self.y
        if key == 2:
            return self.z
        if key == "radius":
            return self.radius
        if key == "width":
            return self.width
        if key == "length":
            return self.length
        if key == "height":
            return self.height



import unittest

from codetocad.enums.preset_landmark import PresetLandmark


class TestLandmarks(unittest.TestCase):
    def test_preset_landmarks(self):
        from codetocad.utilities import (
            min,
            max,
            center,
        )

        assert PresetLandmark.left.get_xyz() == (min, center, center)
        assert PresetLandmark.right.get_xyz() == (max, center, center)
        assert PresetLandmark.leftTop.get_xyz() == (min, center, max)
        assert PresetLandmark.backTop.get_xyz() == (center, max, max)
        assert PresetLandmark.rightBackBottom.get_xyz() == (max, max, min)
        assert PresetLandmark.leftBackTop.get_xyz() == (min, max, max)
        assert PresetLandmark.center.get_xyz() == (center, center, center)



from typing import Optional

from codetocad.core.boundary_axis import BoundaryAxis


class BoundaryBox:
    x: Optional[BoundaryAxis]
    y: Optional[BoundaryAxis]
    z: Optional[BoundaryAxis]

    def __init__(
        self,
        x: Optional[BoundaryAxis],
        y: Optional[BoundaryAxis],
        z: Optional[BoundaryAxis],
    ):
        self.x = x
        self.y = y
        self.z = z

    def __str__(self):
        return f"""    min   max   unit
x   {self.x and self.x.min}  {self.x and self.x.max}  {self.x and self.x.unit and self.x.unit.name+'(s)'}
y   {self.y and self.y.min}  {self.y and self.y.max}  {self.y and self.y.unit and self.y.unit.name+'(s)'}
z   {self.z and self.z.min}  {self.z and self.z.max}  {self.z and self.z.unit and self.z.unit.name+'(s)'}
"""

    def __repr__(self) -> str:
        return self.__str__()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import EntityInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Landmark


class Entity(EntityInterface):
    name: str
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self, name: str, description: Optional[str] = None, native_instance=None
    ):
        self.name = name
        self.description = description
        self.native_instance = native_instance

    def is_exists(self) -> bool:
        print(
            "is_exists called:",
        )
        return True

    def rename(self, new_name: str, renamelinked_entities_and_landmarks: bool = True):
        print("rename called:", new_name, renamelinked_entities_and_landmarks)
        return self

    def delete(self, remove_children: bool = True):
        print("delete called:", remove_children)
        return self

    def is_visible(self) -> bool:
        print(
            "is_visible called:",
        )
        return True

    def set_visible(self, is_visible: bool):
        print("set_visible called:", is_visible)
        return self

    def apply(
        self,
        rotation: bool = True,
        scale: bool = True,
        location: bool = False,
        modifiers: bool = True,
    ):
        print("apply called:", rotation, scale, location, modifiers)
        return self

    def get_native_instance(self) -> object:
        print(
            "get_native_instance called:",
        )
        return "instance"

    def get_location_world(self) -> "Point":
        print(
            "get_location_world called:",
        )
        return Point.from_list_of_float_or_string([0, 0, 0])

    def get_location_local(self) -> "Point":
        print(
            "get_location_local called:",
        )
        return Point.from_list_of_float_or_string([0, 0, 0])

    def select(self):
        print(
            "select called:",
        )
        return self

    def translate_xyz(
        self,
        x: DimensionOrItsFloatOrStringValue,
        y: DimensionOrItsFloatOrStringValue,
        z: DimensionOrItsFloatOrStringValue,
    ):
        print("translate_xyz called:", x, y, z)
        return self

    def translate_x(self, amount: DimensionOrItsFloatOrStringValue):
        print("translate_x called:", amount)
        return self

    def translate_y(self, amount: DimensionOrItsFloatOrStringValue):
        print("translate_y called:", amount)
        return self

    def translate_z(self, amount: DimensionOrItsFloatOrStringValue):
        print("translate_z called:", amount)
        return self

    def rotate_xyz(
        self,
        x: AngleOrItsFloatOrStringValue,
        y: AngleOrItsFloatOrStringValue,
        z: AngleOrItsFloatOrStringValue,
    ):
        print("rotate_xyz called:", x, y, z)
        return self

    def rotate_x(self, rotation: AngleOrItsFloatOrStringValue):
        print("rotate_x called:", rotation)
        return self

    def rotate_y(self, rotation: AngleOrItsFloatOrStringValue):
        print("rotate_y called:", rotation)
        return self

    def rotate_z(self, rotation: AngleOrItsFloatOrStringValue):
        print("rotate_z called:", rotation)
        return self

    def get_bounding_box(self) -> "BoundaryBox":
        print(
            "get_bounding_box called:",
        )
        return BoundaryBox(BoundaryAxis(0, 0), BoundaryAxis(0, 0), BoundaryAxis(0, 0))

    def get_dimensions(self) -> "Dimensions":
        print(
            "get_dimensions called:",
        )
        return Dimensions.from_point(Point.from_list_of_float_or_string([0, 0, 0]))

    def create_landmark(
        self,
        landmark_name: "str",
        x: DimensionOrItsFloatOrStringValue,
        y: DimensionOrItsFloatOrStringValue,
        z: DimensionOrItsFloatOrStringValue,
    ) -> "Landmark":
        from . import Landmark

        print("create_landmark called:", landmark_name, x, y, z)
        return Landmark("name", "parent")

    def get_landmark(self, landmark_name: PresetLandmarkOrItsName) -> "Landmark":
        from . import Landmark

        print("get_landmark called:", landmark_name)
        return Landmark("name", "parent")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from codetocad.interfaces import (
    MirrorableInterface,
    PatternableInterface,
    SubdividableInterface,
    ImportableInterface,
    ExportableInterface,
    ScalableInterface,
)

from . import EntityInterface

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import LandmarkInterface
    from . import EntityInterface
    from . import MaterialInterface


class PartInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    SubdividableInterface,
    ImportableInterface,
    ExportableInterface,
    ScalableInterface,
    metaclass=ABCMeta,
):
    """Capabilities related to creating and manipulating 3D shapes."""

    @abstractmethod
    def create_cube(
        self,
        width: DimensionOrItsFloatOrStringValue,
        length: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        """
        Adds cuboid geometry to a part.
        """

        print(
            "create_cube is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_cone(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        draft_radius: DimensionOrItsFloatOrStringValue = 0,
        keyword_arguments: Optional[dict] = None,
    ):
        """
        Adds cone geometry to a part.
        """

        print(
            "create_cone is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_cylinder(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        """
        Adds cylinder geometry to a part.
        """

        print(
            "create_cylinder is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_torus(
        self,
        inner_radius: DimensionOrItsFloatOrStringValue,
        outer_radius: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        """
        Adds torus geometry to a part.
        """

        print(
            "create_torus is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_sphere(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        """
        Adds sphere geometry to a part.
        """

        print(
            "create_sphere is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_gear(
        self,
        outer_radius: DimensionOrItsFloatOrStringValue,
        addendum: DimensionOrItsFloatOrStringValue,
        inner_radius: DimensionOrItsFloatOrStringValue,
        dedendum: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        pressure_angle: AngleOrItsFloatOrStringValue = "20d",
        number_of_teeth: "int" = 12,
        skew_angle: AngleOrItsFloatOrStringValue = 0,
        conical_angle: AngleOrItsFloatOrStringValue = 0,
        crown_angle: AngleOrItsFloatOrStringValue = 0,
        keyword_arguments: Optional[dict] = None,
    ):
        """
        Adds gear geometry to a part.
        """

        print(
            "create_gear is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def clone(self, new_name: str, copy_landmarks: bool = True) -> "PartInterface":
        """
        Clone an existing Part with its geometry and properties. Returns the new Part.
        """

        print("clone is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def union(
        self,
        with_part: PartOrItsName,
        delete_after_union: bool = True,
        is_transfer_landmarks: bool = False,
    ):
        """
        Boolean union
        """

        print("union is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def subtract(
        self,
        with_part: PartOrItsName,
        delete_after_subtract: bool = True,
        is_transfer_landmarks: bool = False,
    ):
        """
        Boolean subtraction
        """

        print("subtract is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def intersect(
        self,
        with_part: PartOrItsName,
        delete_after_intersect: bool = True,
        is_transfer_landmarks: bool = False,
    ):
        """
        Boolean intersection
        """

        print("intersect is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def hollow(
        self,
        thickness_x: DimensionOrItsFloatOrStringValue,
        thickness_y: DimensionOrItsFloatOrStringValue,
        thickness_z: DimensionOrItsFloatOrStringValue,
        start_axis: AxisOrItsIndexOrItsName = "z",
        flip_axis: bool = False,
    ):
        """
        Remove vertices, if necessary, until the part has a specified wall thickness.
        """

        print("hollow is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def thicken(self, radius: DimensionOrItsFloatOrStringValue):
        """
        Uniformly add a wall around a Part.
        """

        print("thicken is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def hole(
        self,
        hole_landmark: LandmarkOrItsName,
        radius: DimensionOrItsFloatOrStringValue,
        depth: DimensionOrItsFloatOrStringValue,
        normal_axis: AxisOrItsIndexOrItsName = "z",
        flip_axis: bool = False,
        initial_rotation_x: AngleOrItsFloatOrStringValue = 0.0,
        initial_rotation_y: AngleOrItsFloatOrStringValue = 0.0,
        initial_rotation_z: AngleOrItsFloatOrStringValue = 0.0,
        mirror_about_entity_or_landmark: Optional[EntityOrItsName] = None,
        mirror_axis: AxisOrItsIndexOrItsName = "x",
        mirror: bool = False,
        circular_pattern_instance_count: "int" = 1,
        circular_pattern_instance_separation: AngleOrItsFloatOrStringValue = 0.0,
        circular_pattern_instance_axis: AxisOrItsIndexOrItsName = "z",
        circular_pattern_about_entity_or_landmark: Optional[EntityOrItsName] = None,
        linear_pattern_instance_count: "int" = 1,
        linear_pattern_instance_separation: DimensionOrItsFloatOrStringValue = 0.0,
        linear_pattern_instance_axis: AxisOrItsIndexOrItsName = "x",
        linear_pattern2nd_instance_count: "int" = 1,
        linear_pattern2nd_instance_separation: DimensionOrItsFloatOrStringValue = 0.0,
        linear_pattern2nd_instance_axis: AxisOrItsIndexOrItsName = "y",
    ):
        """
        Create a hole.
        """

        print("hole is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def twist(
        self,
        angle: AngleOrItsFloatOrStringValue,
        screw_pitch: DimensionOrItsFloatOrStringValue,
        iterations: "int" = 1,
        axis: AxisOrItsIndexOrItsName = "z",
    ):
        """
        AKA Helix, Screw. Revolve an entity
        """

        print("twist is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def set_material(self, material_name: MaterialOrItsName):
        """
        Assign a known material to this part.
        """

        print(
            "set_material is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def is_colliding_with_part(self, other_part: PartOrItsName) -> bool:
        """
        Check if this part is colliding with another.
        """

        print(
            "is_colliding_with_part is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def fillet_all_edges(
        self, radius: DimensionOrItsFloatOrStringValue, use_width: bool = False
    ):
        """
        Fillet all edges.
        """

        print(
            "fillet_all_edges is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def fillet_edges(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_edges: list[LandmarkOrItsName],
        use_width: bool = False,
    ):
        """
        Fillet specific edges.
        """

        print(
            "fillet_edges is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def fillet_faces(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_faces: list[LandmarkOrItsName],
        use_width: bool = False,
    ):
        """
        Fillet specific faces.
        """

        print(
            "fillet_faces is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def chamfer_all_edges(self, radius: DimensionOrItsFloatOrStringValue):
        """
        Chamfer all edges.
        """

        print(
            "chamfer_all_edges is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def chamfer_edges(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_edges: list[LandmarkOrItsName],
    ):
        """
        Chamfer specific edges.
        """

        print(
            "chamfer_edges is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def chamfer_faces(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_faces: list[LandmarkOrItsName],
    ):
        """
        Chamfer specific faces.
        """

        print(
            "chamfer_faces is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def select_vertex_near_landmark(
        self, landmark_name: Optional[LandmarkOrItsName] = None
    ):
        """
        Select the vertex closest to a Landmark on the entity (in UI).
        """

        print(
            "select_vertex_near_landmark is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def select_edge_near_landmark(
        self, landmark_name: Optional[LandmarkOrItsName] = None
    ):
        """
        Select an edge closest to a landmark on the entity (in UI).
        """

        print(
            "select_edge_near_landmark is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def select_face_near_landmark(
        self, landmark_name: Optional[LandmarkOrItsName] = None
    ):
        """
        Select a face closest to a landmark on the entity (in UI).
        """

        print(
            "select_face_near_landmark is called in an abstract method. Please override this method."
        )
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import VertexInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Entity


class Vertex(Entity, VertexInterface):
    def project(self, project_onto: "Sketch") -> "Projectable":
        from . import Sketch

        print("project called:", project_onto)
        return Sketch("a projected sketch")

    location: "Point"
    parent_entity: Optional[EntityOrItsName] = None
    name: str
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self,
        location: "Point",
        name: str,
        parent_entity: Optional[EntityOrItsName] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        self.location = location
        self.parent_entity = parent_entity
        self.name = name
        self.description = description
        self.native_instance = native_instance

    def get_control_points(self, parameter="") -> "list[Vertex]":
        print("get_control_points called:", parameter)
        return [Vertex(location=(0, 0), name="myVertex")]



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


class ScalableInterface(metaclass=ABCMeta):
    """An entity that can be transformed by a scale."""

    @abstractmethod
    def scale_xyz(
        self,
        x: DimensionOrItsFloatOrStringValue,
        y: DimensionOrItsFloatOrStringValue,
        z: DimensionOrItsFloatOrStringValue,
    ):
        """
        Scale in the XYZ directions. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_xyz is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def scale_x(self, scale: DimensionOrItsFloatOrStringValue):
        """
        Scale in the X direction. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_x is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def scale_y(self, scale: DimensionOrItsFloatOrStringValue):
        """
        Scale in the Y direction. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_y is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def scale_z(self, scale: DimensionOrItsFloatOrStringValue):
        """
        Scale in the Z direction. Pass a number, Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print("scale_z is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def scale_x_by_factor(self, scale_factor: float):
        """
        Scale in the X direction by a multiple.
        """

        print(
            "scale_x_by_factor is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def scale_y_by_factor(self, scale_factor: float):
        """
        Scale in the Y direction by a multiple.
        """

        print(
            "scale_y_by_factor is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def scale_z_by_factor(self, scale_factor: float):
        """
        Scale in the X direction by a multiple.
        """

        print(
            "scale_z_by_factor is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def scale_keep_aspect_ratio(
        self, scale: DimensionOrItsFloatOrStringValue, axis: AxisOrItsIndexOrItsName
    ):
        """
        Scale in one axis and maintain the others. Pass a Dimension or Dimension-String (e.g. '2cm') to scale to a specific length.
        """

        print(
            "scale_keep_aspect_ratio is called in an abstract method. Please override this method."
        )
        return self



# utilities.py contains enums and helper functions for CodeToCAD python functionality.

import re
import sys
from uuid import uuid4
from pathlib import Path

from typing import Optional, Union
from codetocad.codetocad_types import FloatOrItsStringValue
from codetocad.core.angle import Angle
from codetocad.core.boundary_axis import BoundaryAxis
from codetocad.core.boundary_box import BoundaryBox
from codetocad.core.dimension import Dimension

from codetocad.enums.angle_unit import AngleUnit
from codetocad.enums.length_unit import LengthUnit


min = "min"
max = "max"
center = "center"

reservedWords = [min, max, center]


def is_reserved_word_in_string(string_to_check: str) -> bool:
    for word in reservedWords:
        if word in string_to_check:
            return True
    return False


def get_filename(relative_file_path: str):
    path = Path(relative_file_path)
    return path.stem


def get_filenameWithExtension(relative_file_path: str):
    path = Path(relative_file_path)
    return path.name


def get_file_extension(file_path: str):
    path = Path(file_path)
    return path.suffix.replace(".", "")


def get_absolute_filepath(relative_file_path: str):
    path = Path(relative_file_path)
    absoluteFilePath = relative_file_path
    if not path.is_absolute():
        absoluteFilePath = str(Path(sys.argv[0]).parent.joinpath(path).resolve())
    return absoluteFilePath


def create_uuid_like_id():
    return str(uuid4()).replace("-", "")[:10]


def format_landmark_entity_name(parent_entity_name: str, landmark_name: str):
    return f"{parent_entity_name}_{landmark_name}"


def get_angles_from_string_list(angles: Union[str, list[str]]) -> list[Angle]:
    anglesList: list[str]
    if isinstance(angles, str):
        anglesList = angles.replace(" ", "").lower().split(",")
    else:
        anglesList = angles

    assert isinstance(anglesList, (list, tuple)), "Only a list of strings is allowed."

    default_unit: AngleUnit = AngleUnit.DEGREES

    angleString = anglesList[-1]

    if isinstance(angleString, str):
        angleString = angleString.replace(" ", "").lower()
        angleString = re.search("[A-Za-z]+$", angleString)

        unitInString = AngleUnit.from_string(angleString[0]) if angleString else None
        if unitInString is not None and angleString is not None:
            default_unit = unitInString
            if len(angleString[0]) == len(anglesList[-1]):
                anglesList.pop()

    parsedAngles = []
    for angle in anglesList:
        parsedAngles.append(Angle.from_string(angle, default_unit))

    return parsedAngles


# Replace "min|max|center" in "min+0.2" to the value in the bounding box's BoundaryAxis
def replace_min_max_center_with_respective_value(
    dimension: str, boundary_axis: BoundaryAxis, default_unit: LengthUnit
):
    dimension = dimension.lower()

    while "min" in dimension:
        dimension = dimension.replace(
            "min",
            "({})".format(
                Dimension(boundary_axis.min, boundary_axis.unit)
                .convert_to_unit(default_unit)
                .value
            ),
        )
    while "max" in dimension:
        dimension = dimension.replace(
            "max",
            "({})".format(
                Dimension(boundary_axis.max, boundary_axis.unit)
                .convert_to_unit(default_unit)
                .value
            ),
        )
    while "center" in dimension:
        dimension = dimension.replace(
            "center",
            "({})".format(
                Dimension(boundary_axis.center, boundary_axis.unit)
                .convert_to_unit(default_unit)
                .value
            ),
        )

    return dimension


def get_unit_in_string(dimension_string):
    if not isinstance(dimension_string, str):
        return None

    dimension_string = dimension_string.replace(" ", "").lower()

    unitSearchResults = re.search("[A-Za-z]+$", dimension_string)

    unitInString = unitSearchResults[0] if unitSearchResults else None

    return (
        unitInString
        if unitInString and not is_reserved_word_in_string(unitInString)
        else None
    )


def get_dimension_list_from_string_list(
    dimensions: Union[str, list[FloatOrItsStringValue]],
    bounding_box: Optional[BoundaryBox] = None,
) -> list[Dimension]:
    if isinstance(dimensions, str):
        dimensions = dimensions.replace(" ", "").lower().split(",")

    assert isinstance(dimensions, (list, tuple)), "Only a list of strings is allowed."

    parsedDimensions = []

    default_unit = None

    unitInString = get_unit_in_string(dimensions[-1])
    if unitInString is not None:
        default_unit = LengthUnit.from_string(unitInString)
        if len(unitInString) == len(dimensions[-1].replace(" ", "").lower()):
            dimensions.pop()

    for index, dimension in enumerate(dimensions):
        if bounding_box is not None and index < 3:
            boundary_axis = getattr(bounding_box, "xyz"[index])
            parsedDimensions.append(
                Dimension.from_string(dimension, default_unit, boundary_axis)
            )
            continue

        parsedDimensions.append(Dimension.from_string(dimension, default_unit, None))

    return parsedDimensions



# THIS IS AN AUTO-GENERATED FILE. DO NOT CHANGE.

from .entity import Entity
from .part import Part
from .sketch import Sketch
from .vertex import Vertex
from .edge import Edge
from .wire import Wire
from .landmark import Landmark
from .joint import Joint
from .material import Material
from .animation import Animation
from .light import Light
from .camera import Camera
from .render import Render
from .scene import Scene
from .analytics import Analytics



import unittest
from codetocad.core.angle import Angle
from codetocad.core.dimension import Dimension
from codetocad.core.point import Point
from codetocad.core.shapes.circle import get_circle_points, get_point_on_circle_at_angle
from codetocad.core.shapes.clipping import clip_spline_points

from codetocad.core.shapes.ellipse import (
    get_ellipse_points,
    get_ellipse_points_in_fourth_quadrant,
)


class TestUtilities(unittest.TestCase):
    def test_ellipse(self):
        radius_x = Dimension(0.25)
        radius_y = Dimension(0.5)

        # First Quadrant
        quadrant_points = get_ellipse_points_in_fourth_quadrant(
            radius_x, radius_y, int(12 / 4)
        )

        start_point = Point.from_list_of_float_or_string([radius_x * -1, 0, 0])
        end_point = Point.from_list_of_float_or_string([0, radius_y, 0])
        assert len(quadrant_points) == 3, "Unexcepted quadrant length"
        assert quadrant_points[0] == start_point, "Wrong value on curve"
        assert quadrant_points[2] == end_point, "Wrong value on curve"

        ellipse_points = get_ellipse_points(
            radius_x=radius_x, radius_y=radius_y, resolution=64
        )
        assert len(ellipse_points) == 65, "Unexpected ellipse points length"
        assert (
            ellipse_points[0] == ellipse_points[-1]
        ), "First and last points are not equal"

    def test_circle(self):
        radius = Dimension(0.5)

        points = get_circle_points(radius, 64)

        assert points[0] == points[-1], "First and last points are not equal"

    def test_clipping(self):
        radius = Dimension(0.5)

        points = get_circle_points(radius, 4 * 4)

        point_a = get_point_on_circle_at_angle(
            theta_radians=Angle(180 + 45).to_radians().value, radius=radius
        )

        point_b = get_point_on_circle_at_angle(
            theta_radians=Angle(360 - 45).to_radians().value, radius=radius
        )

        clipped_points = clip_spline_points(points, point_a, point_b, is_flip=True)

        assert len(clipped_points) == 15, "Wrong number of clipped points."



from enum import Enum


class ConstraintTypes(Enum):
    # Translation locked between specified start and end points in all axes.
    LimitLocation = 0
    # Rotation locked between specified start and end angles in all axes.
    LimitRotation = 1
    # Rotation locked between specified start and end angles in all axes, but rotation origin is offset.
    Pivot = 2
    # Rotation of one object is a percentage of another's in a specified axis.
    Gear = 3
    # Fixed position:
    FixedPosition = 4
    # Fixed rotation:
    FixedRotation = 5



from enum import Enum


class FileFormats(Enum):
    PNG = 0
    JPEG = 1
    OPEN_EXR = 2
    MP4 = 3

    @staticmethod
    def from_string(name: str):
        name = name.lower()
        if name == "png":
            return FileFormats.PNG
        if name == "jpg" or name == "jpeg":
            return FileFormats.JPEG
        if name == "exr" or name == "openexr":
            return FileFormats.OPEN_EXR
        if name == "mp4" or "ffmpeg":
            return FileFormats.MP4

        raise TypeError(f"{name} is not a supported file type.")



from typing import List
from codetocad.core.dimension import Dimension

from codetocad.core.point import Point


def clip_points_roi(
    points_to_clip: List[Point],
    min_point: Point,
    max_point: Point,
    is_flip: bool = False,
) -> List[Point]:
    """
    Given a list of points, return the points bounded by the region of interest.

    NOTE: The order of items in the list may get changed, so do not use this function if you care about the original sorting of the list.
    """

    clipped_points = []
    clipped_points_flipped = []

    for point in points_to_clip:
        if (point >= min_point) and (point <= max_point):
            clipped_points.append(point)
        else:
            clipped_points_flipped.append(point)

    clipped_points = clipped_points if not is_flip else clipped_points_flipped

    return clipped_points


def clip_spline_points(
    points_to_clip: List[Point],
    point1: Point,
    point2: Point,
    is_flip: bool = False,
    is_include_points: bool = True,
) -> List[Point]:
    """
    Given a list of connected 2D spline points, return the points bounded by two points tangent to the spline.

    If is_include_points flag is True, point1 and point2 will be inserted where the clipping occurs.
    """

    min_point1_distance = float("inf")
    min_point2_distance = float("inf")

    min_point1_index = None
    min_point2_index = None

    for index, point in enumerate(points_to_clip):
        point1_distance = point.distance_to(point1)
        point2_distance = point.distance_to(point2)

        if point1_distance <= min_point1_distance:
            min_point1_distance = point1_distance
            min_point1_index = index

        if point2_distance <= min_point2_distance:
            min_point2_distance = point2_distance
            min_point2_index = index

    min_point = point1
    max_point = point2

    if (min_point1_index or min_point2_index or 0) > (
        min_point2_index or min_point1_index or 0
    ):
        min_point = point2
        max_point = point1

    end_index = len(points_to_clip) - 1
    min_index = min(
        min_point1_index or min_point2_index or 0,
        min_point2_index or min_point1_index or 0,
    )
    max_index = max(
        min_point1_index or min_point2_index or end_index,
        min_point2_index or min_point1_index or end_index,
    )

    clipped_points = points_to_clip[min_index:max_index]

    if is_flip:
        clipped_points = points_to_clip[0:min_index] + points_to_clip[max_index:]

    if is_include_points and is_flip:
        min_index_point = clipped_points[min_index]
        min_index_plus_1_point = clipped_points[min_index + 1]

        if not min_point.is_touching(min_index_point):
            clipped_points.insert(min_index, min_point)
        if not max_point.is_touching(min_index_plus_1_point):
            clipped_points.insert(min_index + 1, max_point)

    elif is_include_points:
        if not min_point.is_touching(clipped_points[0]):
            clipped_points.insert(0, min_point)
        if not max_point.is_touching(clipped_points[-1]):
            clipped_points.append(max_point)

    return clipped_points



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import EntityInterface


class LightInterface(EntityInterface, metaclass=ABCMeta):
    """Manipulate a light object."""

    name: str
    description: Optional[str] = None

    @abstractmethod
    def __init__(
        self, name: str, description: Optional[str] = None, native_instance=None
    ):
        super().__init__(
            name=name, description=description, native_instance=native_instance
        )
        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def set_color(self, r_value: IntOrFloat, g_value: IntOrFloat, b_value: IntOrFloat):
        """
        Set the color of an existing light.
        """

        print("set_color is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def create_sun(self, energy_level: float):
        """
        Create a Sun-type light.
        """

        print(
            "create_sun is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_spot(self, energy_level: float):
        """
        Create a Spot-type light.
        """

        print(
            "create_spot is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_point(self, energy_level: float):
        """
        Create a Point-type light.
        """

        print(
            "create_point is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_area(self, energy_level: float):
        """
        Create an Area-type light.
        """

        print(
            "create_area is called in an abstract method. Please override this method."
        )
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from codetocad.interfaces import (
    MirrorableInterface,
    PatternableInterface,
    ProjectableInterface,
)

from . import EntityInterface

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EdgeInterface
    from . import EntityInterface
    from . import VertexInterface
    from . import PartInterface


class WireInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    ProjectableInterface,
    metaclass=ABCMeta,
):
    """A collection of connected edges."""

    edges: "list[EdgeInterface]"
    parent_entity: Optional[EntityOrItsName] = None

    @abstractmethod
    def __init__(
        self,
        edges: "list[EdgeInterface]",
        name: str,
        parent_entity: Optional[EntityOrItsName] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        super().__init__(
            name=name, description=description, native_instance=native_instance
        )
        self.edges = edges
        self.parent_entity = parent_entity
        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def get_vertices(self) -> "list[VertexInterface]":
        """
        Collapse all edges' vertices into one list.
        """

        print(
            "get_vertices is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def is_closed(self) -> bool:
        """
        A closed wire is a Face.
        """

        print("is_closed is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def loft(
        self, other: "WireInterface", new_part_name: Optional[str] = None
    ) -> "PartInterface":
        """
        Create a surface between two Wires (Faces). If new_part_name is not provided, the two Wires' parents and the surface will be boolean union'ed, and the resulting Part will take the name of the first wire.
        """

        print("loft is called in an abstract method. Please override this method.")
        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import EdgeInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Vertex
    from . import Entity


class Edge(Entity, EdgeInterface):
    def mirror(
        self,
        mirror_across_entity: EntityOrItsName,
        axis: AxisOrItsIndexOrItsName,
        resulting_mirrored_entity_name: Optional[str] = None,
    ):
        print(
            "mirror called:", mirror_across_entity, axis, resulting_mirrored_entity_name
        )
        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: DimensionOrItsFloatOrStringValue,
        direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print("linear_pattern called:", instance_count, offset, direction_axis)
        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: AngleOrItsFloatOrStringValue,
        center_entity_or_landmark: EntityOrItsName,
        normal_direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print(
            "circular_pattern called:",
            instance_count,
            separation_angle,
            center_entity_or_landmark,
            normal_direction_axis,
        )
        return self

    def remesh(self, strategy: str, amount: float):
        print("remesh called:", strategy, amount)
        return self

    def subdivide(self, amount: float):
        print("subdivide called:", amount)
        return self

    def decimate(self, amount: float):
        print("decimate called:", amount)
        return self

    def project(self, project_onto: "Sketch") -> "Projectable":
        from . import Sketch

        print("project called:", project_onto)
        return Sketch("a projected sketch")

    v1: "Vertex"
    v2: "Vertex"
    parent_entity: Optional[EntityOrItsName] = None
    name: str
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self,
        v1: "Vertex",
        v2: "Vertex",
        name: str,
        parent_entity: Optional[EntityOrItsName] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        self.v1 = v1
        self.v2 = v2
        self.parent_entity = parent_entity
        self.name = name
        self.description = description
        self.native_instance = native_instance

    def offset(self, distance: DimensionOrItsFloatOrStringValue) -> "Edge":
        print("offset called:", distance)
        return Edge.get_dummy_edge()

    def fillet(self, other_edge: "Edge", amount: AngleOrItsFloatOrStringValue):
        print("fillet called:", other_edge, amount)
        return self

    def set_is_construction(self, is_construction: bool):
        print("set_is_construction called:", is_construction)
        return self

    def get_is_construction(self) -> bool:
        print(
            "get_is_construction called:",
        )
        return True

    @classmethod
    def get_dummy_edge(cls):
        return cls(v1=(0, 5), v2=(5, 10), name="dummy")



from codetocad.utilities import *
from codetocad.codetocad_types import *
from codetocad.interfaces import *

from importlib import find_loader

if find_loader("blender_provider"):
    from blender_provider import *
else:
    from codetocad.providers_sample import *



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface


class JointInterface(metaclass=ABCMeta):
    """Joints define the relationships and constraints between entities."""

    entity1: EntityOrItsName
    entity2: EntityOrItsName

    @abstractmethod
    def __init__(self, entity1: EntityOrItsName, entity2: EntityOrItsName):
        self.entity1 = entity1
        self.entity2 = entity2

    @abstractmethod
    def translate_landmark_onto_another(self):
        """
        Transforms one landmark onto another
        """

        print(
            "translate_landmark_onto_another is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def pivot(self):
        """
        Constraint the rotation origin of entity B to entity A's landmark.
        """

        print("pivot is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def gear_ratio(self, ratio: float):
        """
        Constraint the rotation of entity B to be a percentage of entity A's
        """

        print(
            "gear_ratio is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_location_xyz(
        self,
        x: Optional[DimensionOrItsFloatOrStringValue] = None,
        y: Optional[DimensionOrItsFloatOrStringValue] = None,
        z: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_xyz is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_location_x(
        self,
        min: Optional[DimensionOrItsFloatOrStringValue] = None,
        max: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_x is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_location_y(
        self,
        min: Optional[DimensionOrItsFloatOrStringValue] = None,
        max: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_y is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_location_z(
        self,
        min: Optional[DimensionOrItsFloatOrStringValue] = None,
        max: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the translation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_location_z is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_rotation_xyz(
        self,
        x: Optional[AngleOrItsFloatOrStringValue] = None,
        y: Optional[AngleOrItsFloatOrStringValue] = None,
        z: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_xyz is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_rotation_x(
        self,
        min: Optional[AngleOrItsFloatOrStringValue] = None,
        max: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_x is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_rotation_y(
        self,
        min: Optional[AngleOrItsFloatOrStringValue] = None,
        max: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_y is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def limit_rotation_z(
        self,
        min: Optional[AngleOrItsFloatOrStringValue] = None,
        max: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        """
        Constraint the rotation of entity B, relative to entity A's landmark.
        """

        print(
            "limit_rotation_z is called in an abstract method. Please override this method."
        )
        return self



from codetocad.utilities import (
    AngleUnit,
    get_angles_from_string_list,
)

import unittest


class TestAngles(unittest.TestCase):
    def test_angles(self):
        angles = get_angles_from_string_list("10,1")
        assert angles[0].value == 10 and angles[1].value == 1
        assert (
            angles[0].unit == AngleUnit.DEGREES and angles[1].unit == AngleUnit.DEGREES
        )
        angles = get_angles_from_string_list("1,2,deg")
        assert angles[0].value == 1 and angles[1].value == 2
        assert (
            angles[0].unit == AngleUnit.DEGREES and angles[1].unit == AngleUnit.DEGREES
        )
        angles = get_angles_from_string_list("1,2,3,deg")
        assert angles[0].value == 1 and angles[1].value == 2 and angles[2].value == 3
        assert (
            angles[0].unit == AngleUnit.DEGREES
            and angles[1].unit == AngleUnit.DEGREES
            and angles[2].unit == AngleUnit.DEGREES
        )
        angles = get_angles_from_string_list("1deg,2rad,3deg")
        assert angles[0].value == 1 and angles[1].value == 2 and angles[2].value == 3
        assert (
            angles[0].unit == AngleUnit.DEGREES
            and angles[1].unit == AngleUnit.RADIANS
            and angles[2].unit == AngleUnit.DEGREES
        )
        angles = get_angles_from_string_list("1,2,3deg,rad")
        assert angles[0].value == 1 and angles[1].value == 2 and angles[2].value == 3
        assert (
            angles[0].unit == AngleUnit.RADIANS
            and angles[1].unit == AngleUnit.RADIANS
            and angles[2].unit == AngleUnit.DEGREES
        )
        angles = get_angles_from_string_list("21,1/8,1/8, degrees")
        assert (
            angles[0].value == 21
            and angles[1].value == 0.125
            and angles[2].value == 0.125
        )
        assert (
            angles[0].unit == AngleUnit.DEGREES
            and angles[1].unit == AngleUnit.DEGREES
            and angles[2].unit == AngleUnit.DEGREES
        )

        print("test_angles done")



from enum import Enum
from typing import Optional


class PresetLandmark(Enum):
    left = 2
    right = 4
    top = 8
    bottom = 16
    front = 32
    back = 64
    center = 128
    leftTop = left | top
    leftBottom = left | bottom
    leftFront = left | front
    leftBack = left | back
    rightTop = right | top
    rightBottom = right | bottom
    rightFront = right | front
    rightBack = right | back
    frontTop = front | top
    backTop = back | top
    frontBottom = front | bottom
    backBottom = back | bottom
    leftFrontTop = left | front | top
    leftBackTop = left | back | top
    leftFrontBottom = left | front | bottom
    leftBackBottom = left | back | bottom
    rightFrontTop = right | front | top
    rightBackTop = right | back | top
    rightFrontBottom = right | front | bottom
    rightBackBottom = right | back | bottom

    def __ror__(self, other):
        return self.value | other.value

    def __and__(self, other):
        return self.value & other.value

    def contains(self, other):
        return self & other == other.value

    @staticmethod
    def from_string(landmark_name) -> Optional["PresetLandmark"]:
        for preset in PresetLandmark:
            if preset.name == landmark_name:
                return preset
        return None

    def get_xyz(self):
        from codetocad.utilities import center, min, max

        x = (
            min
            if self.contains(PresetLandmark.left)
            else max
            if self.contains(PresetLandmark.right)
            else center
        )
        y = (
            min
            if self.contains(PresetLandmark.front)
            else max
            if self.contains(PresetLandmark.back)
            else center
        )
        z = (
            min
            if self.contains(PresetLandmark.bottom)
            else max
            if self.contains(PresetLandmark.top)
            else center
        )
        return (x, y, z)



from enum import Enum
from typing import Union


class Axis(Enum):
    X = 0
    Y = 1
    Z = 2

    @staticmethod
    def from_string(axis: Union[str, float, "Axis"]):
        if isinstance(axis, Axis):
            return axis
        axis = str(axis).lower()
        if axis == "x" or axis == "0":
            return Axis.X
        if axis == "y" or axis == "1":
            return Axis.Y
        if axis == "z" or axis == "2":
            return Axis.Z

        assert False, f"Cannot parse axis {axis}"



import unittest

from codetocad.core.dimension import Dimension
from codetocad.utilities import (
    LengthUnit,
    get_dimension_list_from_string_list,
    BoundaryAxis,
    BoundaryBox,
)


class TestDimensions(unittest.TestCase):
    def test_math(self):
        assert min(Dimension(5), Dimension(3)) == Dimension(
            3
        ), "Unexpected builtin min result"
        assert max(Dimension(5), Dimension(3)) == Dimension(
            5
        ), "Unexpected builtin max result"

    def test_dimensions(self):
        try:
            dimension = Dimension.from_string("")
            assert False
        except Exception as e:
            assert str(e) == "Dimension value cannot be empty."

        try:
            dimension = Dimension.from_string("50NotAUnit")
            assert False
        except Exception as e:
            assert "Could not parse unit" in str(e)

        dimension = Dimension.from_string("50")
        assert dimension.value == 50
        assert dimension.unit is None
        dimension = Dimension.from_string("100")
        assert dimension.value == 100
        assert dimension.unit is None
        dimension = Dimension.from_string("100", LengthUnit.m)
        assert dimension.value == 100
        assert dimension.unit == LengthUnit.m
        dimension = Dimension(99, LengthUnit.m)
        assert dimension.value == 99
        assert dimension.unit == LengthUnit.m

        dimension = Dimension.from_string("100mm")
        assert dimension.value == 100
        assert dimension.unit == LengthUnit.mm
        dimension = Dimension.from_string("1m")
        assert dimension.value == 1
        assert dimension.unit == LengthUnit.m

        dimension = Dimension.from_string("1/4mm")
        assert dimension.value == 0.25
        assert dimension.unit == LengthUnit.mm
        dimension = Dimension.from_string("1-(3/4)")
        assert dimension.value == 0.25
        assert dimension.unit is None
        dimension = Dimension.from_string("1-(3/4)", LengthUnit.ft)
        assert dimension.value == 0.25
        assert dimension.unit == LengthUnit.ft

        dimensions = get_dimension_list_from_string_list("10,1")
        assert dimensions[0].value == 10 and dimensions[1].value == 1
        assert dimensions[0].unit is None and dimensions[1].unit is None
        dimensions = get_dimension_list_from_string_list("1,2,m")
        assert dimensions[0].value == 1 and dimensions[1].value == 2
        assert dimensions[0].unit == LengthUnit.m and dimensions[1].unit == LengthUnit.m
        dimensions = get_dimension_list_from_string_list("1,2,3,m")
        assert (
            dimensions[0].value == 1
            and dimensions[1].value == 2
            and dimensions[2].value == 3
        )
        assert (
            dimensions[0].unit == LengthUnit.m
            and dimensions[1].unit == LengthUnit.m
            and dimensions[2].unit == LengthUnit.m
        )
        dimensions = get_dimension_list_from_string_list("1m,2m,3m")
        assert (
            dimensions[0].value == 1
            and dimensions[1].value == 2
            and dimensions[2].value == 3
        )
        assert (
            dimensions[0].unit == LengthUnit.m
            and dimensions[1].unit == LengthUnit.m
            and dimensions[2].unit == LengthUnit.m
        )
        dimensions = get_dimension_list_from_string_list("1m,2in,3m")
        assert (
            dimensions[0].value == 1
            and dimensions[1].value == 2
            and dimensions[2].value == 3
        )
        assert (
            dimensions[0].unit == LengthUnit.m
            and dimensions[1].unit == LengthUnit.inch
            and dimensions[2].unit == LengthUnit.m
        )
        dimensions = get_dimension_list_from_string_list("1,2,3mm,m")
        assert (
            dimensions[0].value == 1
            and dimensions[1].value == 2
            and dimensions[2].value == 3
        )
        assert (
            dimensions[0].unit == LengthUnit.m
            and dimensions[1].unit == LengthUnit.m
            and dimensions[2].unit == LengthUnit.mm
        )
        dimensions = get_dimension_list_from_string_list("21,1/8,1/8, in")
        assert (
            dimensions[0].value == 21
            and dimensions[1].value == 0.125
            and dimensions[2].value == 0.125
        )
        assert (
            dimensions[0].unit == LengthUnit.inch
            and dimensions[1].unit == LengthUnit.inch
            and dimensions[2].unit == LengthUnit.inch
        )
        dimensions = get_dimension_list_from_string_list("3in,1mm")
        assert dimensions[0].value == 3 and dimensions[1].value == 1
        assert (
            dimensions[0].unit == LengthUnit.inch
            and dimensions[1].unit == LengthUnit.mm
        )

        assert Dimension(1, LengthUnit.m).convert_to_unit(LengthUnit.mm).value == 1000
        assert Dimension(1000, LengthUnit.mm).convert_to_unit(LengthUnit.m).value == 1

        print("test_dimensions done.")

    def test_minMaxCenter(self):
        boundingBox = BoundaryBox(
            BoundaryAxis(-1, 1, LengthUnit.m),
            BoundaryAxis(-1, 1, LengthUnit.m),
            BoundaryAxis(-1, 1, LengthUnit.m),
        )

        dimensions = get_dimension_list_from_string_list("min", boundingBox)
        assert dimensions[0].value == -1
        assert dimensions[0].unit == LengthUnit.m
        dimensions = get_dimension_list_from_string_list("max", boundingBox)
        assert dimensions[0].value == 1
        assert dimensions[0].unit == LengthUnit.m
        dimensions = get_dimension_list_from_string_list("center", boundingBox)
        assert dimensions[0].value == 0
        assert dimensions[0].unit == LengthUnit.m
        dimensions = get_dimension_list_from_string_list("min-2", boundingBox)
        assert dimensions[0].value == -3
        assert dimensions[0].unit == LengthUnit.m
        dimensions = get_dimension_list_from_string_list("min-2cm", boundingBox)
        assert dimensions[0].value == -102
        assert dimensions[0].unit == LengthUnit.cm

        print("test_minMaxCenter() done.")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import PartInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Landmark
    from . import Entity
    from . import Material


class Part(Entity, PartInterface):
    def mirror(
        self,
        mirror_across_entity: EntityOrItsName,
        axis: AxisOrItsIndexOrItsName,
        resulting_mirrored_entity_name: Optional[str] = None,
    ):
        print(
            "mirror called:", mirror_across_entity, axis, resulting_mirrored_entity_name
        )
        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: DimensionOrItsFloatOrStringValue,
        direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print("linear_pattern called:", instance_count, offset, direction_axis)
        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: AngleOrItsFloatOrStringValue,
        center_entity_or_landmark: EntityOrItsName,
        normal_direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print(
            "circular_pattern called:",
            instance_count,
            separation_angle,
            center_entity_or_landmark,
            normal_direction_axis,
        )
        return self

    def remesh(self, strategy: str, amount: float):
        print("remesh called:", strategy, amount)
        return self

    def subdivide(self, amount: float):
        print("subdivide called:", amount)
        return self

    def decimate(self, amount: float):
        print("decimate called:", amount)
        return self

    def create_from_file(self, file_path: str, file_type: Optional[str] = None):
        print("create_from_file called:", file_path, file_type)
        return self

    def export(self, file_path: str, overwrite: bool = True, scale: float = 1.0):
        print("export called:", file_path, overwrite, scale)
        return self

    def scale_xyz(
        self,
        x: DimensionOrItsFloatOrStringValue,
        y: DimensionOrItsFloatOrStringValue,
        z: DimensionOrItsFloatOrStringValue,
    ):
        print("scale_xyz called:", x, y, z)
        return self

    def scale_x(self, scale: DimensionOrItsFloatOrStringValue):
        print("scale_x called:", scale)
        return self

    def scale_y(self, scale: DimensionOrItsFloatOrStringValue):
        print("scale_y called:", scale)
        return self

    def scale_z(self, scale: DimensionOrItsFloatOrStringValue):
        print("scale_z called:", scale)
        return self

    def scale_x_by_factor(self, scale_factor: float):
        print("scale_x_by_factor called:", scale_factor)
        return self

    def scale_y_by_factor(self, scale_factor: float):
        print("scale_y_by_factor called:", scale_factor)
        return self

    def scale_z_by_factor(self, scale_factor: float):
        print("scale_z_by_factor called:", scale_factor)
        return self

    def scale_keep_aspect_ratio(
        self, scale: DimensionOrItsFloatOrStringValue, axis: AxisOrItsIndexOrItsName
    ):
        print("scale_keep_aspect_ratio called:", scale, axis)
        return self

    def create_cube(
        self,
        width: DimensionOrItsFloatOrStringValue,
        length: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        print("create_cube called:", width, length, height, keyword_arguments)
        return self

    def create_cone(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        draft_radius: DimensionOrItsFloatOrStringValue = 0,
        keyword_arguments: Optional[dict] = None,
    ):
        print("create_cone called:", radius, height, draft_radius, keyword_arguments)
        return self

    def create_cylinder(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        print("create_cylinder called:", radius, height, keyword_arguments)
        return self

    def create_torus(
        self,
        inner_radius: DimensionOrItsFloatOrStringValue,
        outer_radius: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        print("create_torus called:", inner_radius, outer_radius, keyword_arguments)
        return self

    def create_sphere(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        keyword_arguments: Optional[dict] = None,
    ):
        print("create_sphere called:", radius, keyword_arguments)
        return self

    def create_gear(
        self,
        outer_radius: DimensionOrItsFloatOrStringValue,
        addendum: DimensionOrItsFloatOrStringValue,
        inner_radius: DimensionOrItsFloatOrStringValue,
        dedendum: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
        pressure_angle: AngleOrItsFloatOrStringValue = "20d",
        number_of_teeth: "int" = 12,
        skew_angle: AngleOrItsFloatOrStringValue = 0,
        conical_angle: AngleOrItsFloatOrStringValue = 0,
        crown_angle: AngleOrItsFloatOrStringValue = 0,
        keyword_arguments: Optional[dict] = None,
    ):
        print(
            "create_gear called:",
            outer_radius,
            addendum,
            inner_radius,
            dedendum,
            height,
            pressure_angle,
            number_of_teeth,
            skew_angle,
            conical_angle,
            crown_angle,
            keyword_arguments,
        )
        return self

    def clone(self, new_name: str, copy_landmarks: bool = True) -> "Part":
        print("clone called:", new_name, copy_landmarks)
        return Part("a part")

    def union(
        self,
        with_part: PartOrItsName,
        delete_after_union: bool = True,
        is_transfer_landmarks: bool = False,
    ):
        print("union called:", with_part, delete_after_union, is_transfer_landmarks)
        return self

    def subtract(
        self,
        with_part: PartOrItsName,
        delete_after_subtract: bool = True,
        is_transfer_landmarks: bool = False,
    ):
        print(
            "subtract called:", with_part, delete_after_subtract, is_transfer_landmarks
        )
        return self

    def intersect(
        self,
        with_part: PartOrItsName,
        delete_after_intersect: bool = True,
        is_transfer_landmarks: bool = False,
    ):
        print(
            "intersect called:",
            with_part,
            delete_after_intersect,
            is_transfer_landmarks,
        )
        return self

    def hollow(
        self,
        thickness_x: DimensionOrItsFloatOrStringValue,
        thickness_y: DimensionOrItsFloatOrStringValue,
        thickness_z: DimensionOrItsFloatOrStringValue,
        start_axis: AxisOrItsIndexOrItsName = "z",
        flip_axis: bool = False,
    ):
        print(
            "hollow called:",
            thickness_x,
            thickness_y,
            thickness_z,
            start_axis,
            flip_axis,
        )
        return self

    def thicken(self, radius: DimensionOrItsFloatOrStringValue):
        print("thicken called:", radius)
        return self

    def hole(
        self,
        hole_landmark: LandmarkOrItsName,
        radius: DimensionOrItsFloatOrStringValue,
        depth: DimensionOrItsFloatOrStringValue,
        normal_axis: AxisOrItsIndexOrItsName = "z",
        flip_axis: bool = False,
        initial_rotation_x: AngleOrItsFloatOrStringValue = 0.0,
        initial_rotation_y: AngleOrItsFloatOrStringValue = 0.0,
        initial_rotation_z: AngleOrItsFloatOrStringValue = 0.0,
        mirror_about_entity_or_landmark: Optional[EntityOrItsName] = None,
        mirror_axis: AxisOrItsIndexOrItsName = "x",
        mirror: bool = False,
        circular_pattern_instance_count: "int" = 1,
        circular_pattern_instance_separation: AngleOrItsFloatOrStringValue = 0.0,
        circular_pattern_instance_axis: AxisOrItsIndexOrItsName = "z",
        circular_pattern_about_entity_or_landmark: Optional[EntityOrItsName] = None,
        linear_pattern_instance_count: "int" = 1,
        linear_pattern_instance_separation: DimensionOrItsFloatOrStringValue = 0.0,
        linear_pattern_instance_axis: AxisOrItsIndexOrItsName = "x",
        linear_pattern2nd_instance_count: "int" = 1,
        linear_pattern2nd_instance_separation: DimensionOrItsFloatOrStringValue = 0.0,
        linear_pattern2nd_instance_axis: AxisOrItsIndexOrItsName = "y",
    ):
        print(
            "hole called:",
            hole_landmark,
            radius,
            depth,
            normal_axis,
            flip_axis,
            initial_rotation_x,
            initial_rotation_y,
            initial_rotation_z,
            mirror_about_entity_or_landmark,
            mirror_axis,
            mirror,
            circular_pattern_instance_count,
            circular_pattern_instance_separation,
            circular_pattern_instance_axis,
            circular_pattern_about_entity_or_landmark,
            linear_pattern_instance_count,
            linear_pattern_instance_separation,
            linear_pattern_instance_axis,
            linear_pattern2nd_instance_count,
            linear_pattern2nd_instance_separation,
            linear_pattern2nd_instance_axis,
        )
        return self

    def twist(
        self,
        angle: AngleOrItsFloatOrStringValue,
        screw_pitch: DimensionOrItsFloatOrStringValue,
        iterations: "int" = 1,
        axis: AxisOrItsIndexOrItsName = "z",
    ):
        print("twist called:", angle, screw_pitch, iterations, axis)
        return self

    def set_material(self, material_name: MaterialOrItsName):
        print("set_material called:", material_name)
        return self

    def is_colliding_with_part(self, other_part: PartOrItsName) -> bool:
        print("is_colliding_with_part called:", other_part)
        return True

    def fillet_all_edges(
        self, radius: DimensionOrItsFloatOrStringValue, use_width: bool = False
    ):
        print("fillet_all_edges called:", radius, use_width)
        return self

    def fillet_edges(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_edges: list[LandmarkOrItsName],
        use_width: bool = False,
    ):
        print("fillet_edges called:", radius, landmarks_near_edges, use_width)
        return self

    def fillet_faces(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_faces: list[LandmarkOrItsName],
        use_width: bool = False,
    ):
        print("fillet_faces called:", radius, landmarks_near_faces, use_width)
        return self

    def chamfer_all_edges(self, radius: DimensionOrItsFloatOrStringValue):
        print("chamfer_all_edges called:", radius)
        return self

    def chamfer_edges(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_edges: list[LandmarkOrItsName],
    ):
        print("chamfer_edges called:", radius, landmarks_near_edges)
        return self

    def chamfer_faces(
        self,
        radius: DimensionOrItsFloatOrStringValue,
        landmarks_near_faces: list[LandmarkOrItsName],
    ):
        print("chamfer_faces called:", radius, landmarks_near_faces)
        return self

    def select_vertex_near_landmark(
        self, landmark_name: Optional[LandmarkOrItsName] = None
    ):
        print("select_vertex_near_landmark called:", landmark_name)
        return self

    def select_edge_near_landmark(
        self, landmark_name: Optional[LandmarkOrItsName] = None
    ):
        print("select_edge_near_landmark called:", landmark_name)
        return self

    def select_face_near_landmark(
        self, landmark_name: Optional[LandmarkOrItsName] = None
    ):
        print("select_face_near_landmark called:", landmark_name)
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import SketchInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Part
    from . import Entity
    from . import Wire
    from . import Vertex
    from . import Edge


class Sketch(Entity, SketchInterface):
    def mirror(
        self,
        mirror_across_entity: EntityOrItsName,
        axis: AxisOrItsIndexOrItsName,
        resulting_mirrored_entity_name: Optional[str] = None,
    ):
        print(
            "mirror called:", mirror_across_entity, axis, resulting_mirrored_entity_name
        )
        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: DimensionOrItsFloatOrStringValue,
        direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print("linear_pattern called:", instance_count, offset, direction_axis)
        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: AngleOrItsFloatOrStringValue,
        center_entity_or_landmark: EntityOrItsName,
        normal_direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print(
            "circular_pattern called:",
            instance_count,
            separation_angle,
            center_entity_or_landmark,
            normal_direction_axis,
        )
        return self

    def create_from_file(self, file_path: str, file_type: Optional[str] = None):
        print("create_from_file called:", file_path, file_type)
        return self

    def export(self, file_path: str, overwrite: bool = True, scale: float = 1.0):
        print("export called:", file_path, overwrite, scale)
        return self

    def scale_xyz(
        self,
        x: DimensionOrItsFloatOrStringValue,
        y: DimensionOrItsFloatOrStringValue,
        z: DimensionOrItsFloatOrStringValue,
    ):
        print("scale_xyz called:", x, y, z)
        return self

    def scale_x(self, scale: DimensionOrItsFloatOrStringValue):
        print("scale_x called:", scale)
        return self

    def scale_y(self, scale: DimensionOrItsFloatOrStringValue):
        print("scale_y called:", scale)
        return self

    def scale_z(self, scale: DimensionOrItsFloatOrStringValue):
        print("scale_z called:", scale)
        return self

    def scale_x_by_factor(self, scale_factor: float):
        print("scale_x_by_factor called:", scale_factor)
        return self

    def scale_y_by_factor(self, scale_factor: float):
        print("scale_y_by_factor called:", scale_factor)
        return self

    def scale_z_by_factor(self, scale_factor: float):
        print("scale_z_by_factor called:", scale_factor)
        return self

    def scale_keep_aspect_ratio(
        self, scale: DimensionOrItsFloatOrStringValue, axis: AxisOrItsIndexOrItsName
    ):
        print("scale_keep_aspect_ratio called:", scale, axis)
        return self

    name: str
    curve_type: Optional["CurveTypes"] = None
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self,
        name: str,
        curve_type: Optional["CurveTypes"] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        self.name = name
        self.curve_type = curve_type
        self.description = description
        self.native_instance = native_instance

    def clone(self, new_name: str, copy_landmarks: bool = True) -> "Sketch":
        print("clone called:", new_name, copy_landmarks)
        return Sketch("a sketch")

    def revolve(
        self,
        angle: AngleOrItsFloatOrStringValue,
        about_entity_or_landmark: EntityOrItsName,
        axis: AxisOrItsIndexOrItsName = "z",
    ) -> "Part":
        from . import Part

        print("revolve called:", angle, about_entity_or_landmark, axis)
        return Part("a part")

    def twist(
        self,
        angle: AngleOrItsFloatOrStringValue,
        screw_pitch: DimensionOrItsFloatOrStringValue,
        iterations: "int" = 1,
        axis: AxisOrItsIndexOrItsName = "z",
    ):
        print("twist called:", angle, screw_pitch, iterations, axis)
        return self

    def extrude(self, length: DimensionOrItsFloatOrStringValue) -> "Part":
        from . import Part

        print("extrude called:", length)
        return Part("a part")

    def sweep(
        self, profile_name_or_instance: SketchOrItsName, fill_cap: bool = True
    ) -> "Part":
        from . import Part

        print("sweep called:", profile_name_or_instance, fill_cap)
        return Part("a part")

    def offset(self, radius: DimensionOrItsFloatOrStringValue):
        print("offset called:", radius)
        return self

    def profile(self, profile_curve_name: str):
        print("profile called:", profile_curve_name)
        return self

    def create_text(
        self,
        text: str,
        font_size: DimensionOrItsFloatOrStringValue = 1.0,
        bold: bool = False,
        italic: bool = False,
        underlined: bool = False,
        character_spacing: "int" = 1,
        word_spacing: "int" = 1,
        line_spacing: "int" = 1,
        font_file_path: Optional[str] = None,
    ):
        print(
            "create_text called:",
            text,
            font_size,
            bold,
            italic,
            underlined,
            character_spacing,
            word_spacing,
            line_spacing,
            font_file_path,
        )
        return self

    def create_from_vertices(
        self, points: list[PointOrListOfFloatOrItsStringValueOrVertex]
    ) -> "Wire":
        print("create_from_vertices called:", points)
        return None

    def create_point(self, point: PointOrListOfFloatOrItsStringValue) -> "Vertex":
        print("create_point called:", point)
        return None

    def create_line(
        self,
        start_at: PointOrListOfFloatOrItsStringValueOrVertex,
        end_at: PointOrListOfFloatOrItsStringValueOrVertex,
    ) -> "Edge":
        from . import Edge

        print("create_line called:", start_at, end_at)
        return Edge.get_dummy_edge()

    def create_circle(self, radius: DimensionOrItsFloatOrStringValue) -> "Wire":
        from . import Wire

        print("create_circle called:", radius)
        return Wire(
            edges=[],
            name="wire",
            parent_entity="myEdge",
        )

    def create_ellipse(
        self,
        radius_minor: DimensionOrItsFloatOrStringValue,
        radius_major: DimensionOrItsFloatOrStringValue,
    ) -> "Wire":
        from . import Wire

        print("create_ellipse called:", radius_minor, radius_major)
        return Wire(
            edges=[],
            name="wire",
            parent_entity="myEdge",
        )

    def create_arc(
        self,
        start_at: PointOrListOfFloatOrItsStringValueOrVertex,
        end_at: PointOrListOfFloatOrItsStringValueOrVertex,
        radius: DimensionOrItsFloatOrStringValue,
        flip: Optional[bool] = False,
    ) -> "Wire":
        print("create_arc called:", start_at, end_at, radius, flip)
        return None

    def create_rectangle(
        self,
        length: DimensionOrItsFloatOrStringValue,
        width: DimensionOrItsFloatOrStringValue,
    ) -> "Wire":
        print("create_rectangle called:", length, width)
        return None

    def create_polygon(
        self,
        number_of_sides: "int",
        length: DimensionOrItsFloatOrStringValue,
        width: DimensionOrItsFloatOrStringValue,
    ) -> "Wire":
        from . import Wire, Edge

        print("create_polygon called:", number_of_sides, length, width)
        return Wire(edges=[Edge(v1=(0, 0), v2=(5, 5), name="myEdge")], name="myWire")

    def create_trapezoid(
        self,
        length_upper: DimensionOrItsFloatOrStringValue,
        length_lower: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
    ) -> "Wire":
        from . import Wire, Edge

        print("create_trapezoid called:", length_upper, length_lower, height)
        return Wire(edges=[Edge(v1=(0, 0), v2=(5, 5), name="myEdge")], name="myWire")

    def create_spiral(
        self,
        number_of_turns: "int",
        height: DimensionOrItsFloatOrStringValue,
        radius: DimensionOrItsFloatOrStringValue,
        is_clockwise: bool = True,
        radius_end: Optional[DimensionOrItsFloatOrStringValue] = None,
    ) -> "Wire":
        from . import Wire, Edge

        print(
            "create_spiral called:",
            number_of_turns,
            height,
            radius,
            is_clockwise,
            radius_end,
        )
        return Wire(edges=[Edge(v1=(0, 0), v2=(5, 5), name="myEdge")], name="myWire")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from codetocad.interfaces import (
    MirrorableInterface,
    PatternableInterface,
    SubdividableInterface,
    ProjectableInterface,
)

from . import EntityInterface

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import VertexInterface
    from . import EntityInterface


class EdgeInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    SubdividableInterface,
    ProjectableInterface,
    metaclass=ABCMeta,
):
    """A curve bounded by two Vertices."""

    v1: "VertexInterface"
    v2: "VertexInterface"
    parent_entity: Optional[EntityOrItsName] = None

    @abstractmethod
    def __init__(
        self,
        v1: "VertexInterface",
        v2: "VertexInterface",
        name: str,
        parent_entity: Optional[EntityOrItsName] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        super().__init__(
            name=name, description=description, native_instance=native_instance
        )
        self.v1 = v1
        self.v2 = v2
        self.parent_entity = parent_entity
        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def offset(self, distance: DimensionOrItsFloatOrStringValue) -> "EdgeInterface":
        """
        Clone and offset this edge a distance away from this one.
        """

        print("offset is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def fillet(self, other_edge: "EdgeInterface", amount: AngleOrItsFloatOrStringValue):
        """
        Fillet this and another edge.
        """

        print("fillet is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def set_is_construction(self, is_construction: bool):
        """
        Mark this edge for construction only.
        """

        print(
            "set_is_construction is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def get_is_construction(self) -> bool:
        """
        Check if this edge is for construction only.
        """

        print(
            "get_is_construction is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()



import math
import re
from typing import Union

from codetocad.enums.angle_unit import AngleUnit


class Angle:
    def to_radians(self) -> "Angle":
        return Angle(
            math.radians(self.value) if self.unit == AngleUnit.DEGREES else self.value,
            AngleUnit.RADIANS,
        )

    def to_degrees(self) -> "Angle":
        return Angle(
            math.degrees(self.value) if self.unit == AngleUnit.RADIANS else self.value,
            AngleUnit.DEGREES,
        )

    # Default unit is degrees if unit not passed
    def __init__(
        self, value: float, default_unit: AngleUnit = AngleUnit.DEGREES
    ) -> None:
        unit = (
            AngleUnit.from_string(default_unit.replace(" ", "").lower())
            if isinstance(default_unit, str)
            else default_unit
        )
        assert (unit is None and default_unit is None) or isinstance(
            unit, AngleUnit
        ), "Could not parse default unit."

        self.value = value
        self.unit = unit or AngleUnit.DEGREES

    @staticmethod
    def from_angle_or_its_float_or_string_value(
        mystery_angle: Union[str, float, "Angle"]
    ) -> "Angle":
        if isinstance(mystery_angle, Angle):
            return mystery_angle
        if isinstance(mystery_angle, (int, float)):
            return Angle(mystery_angle)
        return Angle.from_string(mystery_angle)

    def __str__(self) -> str:
        return f"{self.value}{' '+self.unit.name.lower() if self.unit else ''}"

    def __repr__(self) -> str:
        return self.__str__()

    def arithmetic_precheck_and_unit_conversion(self, other):
        if not isinstance(other, Angle):
            other = Angle.from_string(other)
        if other.unit != self.unit:
            if self.unit == AngleUnit.DEGREES:
                other.to_degrees()
            else:
                other.to_radians()
        return other

    def __add__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Angle(self.value + other.value, self.unit)

    def __sub__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Angle(self.value - other.value, self.unit)

    def __mul__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Angle(self.value * other.value, self.unit)

    def __truediv__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Angle(self.value / other.value, self.unit)

    def __floordiv__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Angle(self.value // other.value, self.unit)

    def __mod__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Angle(self.value % other.value, self.unit)

    # def __divmod__(self, other):
    #     other = self.arithmetic_precheck_and_unit_conversion(other)
    #     return Angle(divmod(self.value, other.value), self.unit)

    def __pow__(self, other, mod=None):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        return Angle(pow(self.value, other.value, mod), self.unit)

    def __abs__(self):
        return Angle(abs(self.value), self.unit)

    def __lt__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value < other.value

    def __le__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value <= other.value

    def __gt__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value > other.value

    def __ge__(self, other):
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value >= other.value

    def __eq__(self, other):
        if other is None:
            return False
        other = self.arithmetic_precheck_and_unit_conversion(other)
        assert self.unit == other.unit, "Units are not matching for comparison."
        return self.value == other.value

    def __ne__(self, other):
        return not (self == other)

    def __copy__(self):
        return self.__deepcopy__()

    def __deepcopy__(self):
        return Angle(self.value, self.unit)

    def copy(self):
        return self.__deepcopy__()

    # from_string: takes a string with a math operation and an optional unit of measurement
    # Default unit is degrees if unit not passed
    @staticmethod
    def from_string(
        from_string: Union[str, float, "Angle"],
        default_unit: Union[str, AngleUnit] = AngleUnit.DEGREES,
    ):
        if isinstance(from_string, Angle):
            return from_string.copy()

        unit = (
            AngleUnit.from_string(default_unit.replace(" ", "").lower())
            if isinstance(default_unit, str)
            else default_unit
        )
        assert (unit is None and default_unit is None) or isinstance(
            unit, AngleUnit
        ), "Could not parse default unit."

        if isinstance(from_string, (int, float)):
            return Angle(from_string, unit)

        assert isinstance(from_string, str), "from_string must be a string."

        from_string = from_string.replace(" ", "").lower()

        value = from_string

        assert len(value) > 0, "Angle value cannot be empty."

        # check if a unit is passed into from_string, e.g. "1rad" -> radians
        unitInString = re.search("[A-Za-z]+$", from_string)
        if unitInString:
            value = from_string[0 : -1 * len(unitInString[0])]
            unitInString = AngleUnit.from_string(unitInString[0])
            unit = unitInString or unit or AngleUnit.DEGREES

        # Make sure our value only contains math operations and numbers as a weak safety check before passing it to `eval`
        assert re.match(
            r"[+\-*\/%\d\(\)]+", value
        ), f"Value {value} contains characters that are not allowed."

        value = eval(value)

        return Angle(value, unit)



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import MaterialInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Part


class Material(MaterialInterface):
    name: str
    description: Optional[str] = None

    def __init__(self, name: str, description: Optional[str] = None):
        self.name = name
        self.description = description

    def assign_to_part(self, part_name_or_instance: PartOrItsName):
        print("assign_to_part called:", part_name_or_instance)
        return self

    def set_color(
        self,
        r_value: IntOrFloat,
        g_value: IntOrFloat,
        b_value: IntOrFloat,
        a_value: IntOrFloat = 1.0,
    ):
        print("set_color called:", r_value, g_value, b_value, a_value)
        return self

    def set_reflectivity(self, reflectivity: float):
        print("set_reflectivity called:", reflectivity)
        return self

    def set_roughness(self, roughness: float):
        print("set_roughness called:", roughness)
        return self

    def set_image_texture(self, image_file_path: str):
        print("set_image_texture called:", image_file_path)
        return self

    @classmethod
    def get_sample_mat_instance(cls):
        return cls(name="myMaterial", description="material is created for testing")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import SceneInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Entity
    from . import Exportable


class Scene(SceneInterface):
    name: Optional[str] = None
    description: Optional[str] = None

    def __init__(self, name: Optional[str] = None, description: Optional[str] = None):
        self.name = name
        self.description = description

    @staticmethod
    def default() -> "Scene":
        return Scene()

    def create(self):
        print(
            "create called:",
        )
        return self

    def delete(self):
        print(
            "delete called:",
        )
        return self

    def get_selected_entity(self) -> "Entity":
        from . import Entity

        print(
            "get_selected_entity called:",
        )
        return Entity("an entity")

    def export(
        self,
        file_path: str,
        entities: "list[string,Exportable]",
        overwrite: bool = True,
        scale: float = 1.0,
    ):
        print("export called:", file_path, entities, overwrite, scale)
        return self

    def set_default_unit(self, unit: LengthUnitOrItsName):
        print("set_default_unit called:", unit)
        return self

    def create_group(self, name: str):
        print("create_group called:", name)
        return self

    def delete_group(self, name: str, remove_children: bool):
        print("delete_group called:", name, remove_children)
        return self

    def remove_from_group(self, entity_name: str, group_name: str):
        print("remove_from_group called:", entity_name, group_name)
        return self

    def assign_to_group(
        self,
        entities: list[EntityOrItsName],
        group_name: str,
        remove_from_other_groups: Optional[bool] = True,
    ):
        print("assign_to_group called:", entities, group_name, remove_from_other_groups)
        return self

    def set_visible(self, entities: list[EntityOrItsName], is_visible: bool):
        print("set_visible called:", entities, is_visible)
        return self

    def set_background_image(
        self,
        file_path: str,
        location_x: Optional[DimensionOrItsFloatOrStringValue] = 0,
        location_y: Optional[DimensionOrItsFloatOrStringValue] = 0,
    ):
        print("set_background_image called:", file_path, location_x, location_y)
        return self

    @classmethod
    def get_sample_scene(
        cls,
    ):
        return cls(name="myScene", description="this is for testing purpose")

    def is_exists(self):
        print("is_exists called")
        return True
